package il.ac.tau.cs.sw1.addressbook.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Sash;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

public class AddressBookView {

	private Label name, email, phone, street, city, zipcode, country;
	private final Shell shell;
	private Table table;

	// TODO: Keep a reference to an underlying address book

	public AddressBookView() {
		shell = createShell();
	}
	
	/**
	 * @return the shell
	 */
	public Shell getShell() {
		return shell;
	}

	/**
	 * Creates the window for the application. The window contains two parts,
	 * the top section is a table listing all contacts in the address book. The
	 * bottom part displays the details of a contact once it has been selected.
	 * creation of the contact details view is done in the
	 * createContactDetailsView method.
	 */
	private Shell createShell() {
		Shell shell = new Shell();
		shell.setText("Address Book");

		Rectangle monitor_bounds = shell.getMonitor().getBounds();
		shell.setSize(new Point(monitor_bounds.width / 3,
				monitor_bounds.height / 2));

		Menu menu = new Menu(shell, SWT.BAR);
		shell.setMenuBar(menu);

		MenuItem fileItem = new MenuItem(menu, SWT.CASCADE);
		MenuItem contactsItem = new MenuItem(menu, SWT.CASCADE);
		MenuItem helpItem = new MenuItem(menu, SWT.CASCADE);

		fileItem.setText("&File");
		contactsItem.setText("&Contacts");
		helpItem.setText("&Help");

		Menu fileMenu = new Menu(shell, SWT.DROP_DOWN);
		fileItem.setMenu(fileMenu);
		Menu contactsMenu = new Menu(shell, SWT.DROP_DOWN);
		contactsItem.setMenu(contactsMenu);
		Menu helpMemu = new Menu(shell, SWT.DROP_DOWN);
		helpItem.setMenu(helpMemu);

		MenuItem newItem = new MenuItem(fileMenu, SWT.PUSH);
		newItem.setText("New &Address Book\tCtrl+N");
		newItem.setAccelerator(SWT.MOD1 + 'N');
		// TODO: add listener to handle 'new address book' event

		MenuItem openItem = new MenuItem(fileMenu, SWT.PUSH);
		openItem.setText("&Open\tCtrl+O");
		openItem.setAccelerator(SWT.MOD1 + 'O');
		// TODO: add listener to handle 'open address book' event

		MenuItem saveItem = new MenuItem(fileMenu, SWT.PUSH);
		saveItem.setText("&Save\tCtrl+S");
		saveItem.setAccelerator(SWT.MOD1 + 'S');
		// TODO: add listener to handle 'save address book' event

		MenuItem exitItem = new MenuItem(fileMenu, SWT.PUSH);
		exitItem.setText("E&xit\tCtrl+Q");
		exitItem.setAccelerator(SWT.MOD1 + 'Q');
		// TODO: add listener to handle 'exit' event

		MenuItem newContactItem = new MenuItem(contactsMenu, SWT.PUSH);
		newContactItem.setText("New &Contact\tCtrl+M");
		newContactItem.setAccelerator(SWT.MOD1 + 'M');
		// TODO: add listener to handle 'new contact' event

		MenuItem editItem = new MenuItem(contactsMenu, SWT.PUSH);
		editItem.setText("&Edit");
		// TODO: add listener to handle 'edit contact' event

		MenuItem deleteItem = new MenuItem(contactsMenu, SWT.PUSH);
		deleteItem.setText("&Delete");
		deleteItem.setAccelerator(SWT.DEL);
		// TODO: add listener to handle 'delete contact' event

		createContactDetailsView(shell);
		return shell;
	}

	/**
	 * This method creates a simple
	 * 
	 * @param parent
	 */
	private void createContactDetailsView(final Composite parent) {
		table = new Table(parent, SWT.BORDER | SWT.SINGLE | SWT.FULL_SELECTION);

		TableColumn nameColumn = new TableColumn(table, SWT.NONE);
		nameColumn.setText("Name");
		TableColumn emailColumn = new TableColumn(table, SWT.NONE);
		emailColumn.setText("Email");
		TableColumn phoneColumn = new TableColumn(table, SWT.NONE);
		phoneColumn.setText("Phone Number");

		table.setHeaderVisible(true);

		int tableFullWidth = parent.getClientArea().width
				- table.getBorderWidth() * 2;
		nameColumn.setWidth((int) (tableFullWidth * 0.20));
		emailColumn.setWidth((int) (tableFullWidth * 0.40));
		// Implementation Note: It is more precise to set the width of
		// phoneColumn to be "tableFullWidth - nameColumn.getWidth() - emailColumn.getWidth()"
		// instead of (int)(fullWidth * 0.40)
		phoneColumn.setWidth(tableFullWidth - nameColumn.getWidth()
				- emailColumn.getWidth());

		Composite form = new Composite(parent, SWT.BORDER);
		form.setLayout(new GridLayout(2, false));

		Label nameLabel = new Label(form, SWT.NONE);
		nameLabel.setText("Name");
		nameLabel.setLayoutData(new GridData(GridData.BEGINNING,
				GridData.CENTER, false, false));
		name = new Label(form, SWT.SINGLE | SWT.BORDER);
		name.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true,
				false));

		Label emailLable = new Label(form, SWT.NONE);
		emailLable.setText("Email");
		emailLable.setLayoutData(new GridData(GridData.BEGINNING,
				GridData.CENTER, false, false));
		email = new Label(form, SWT.SINGLE | SWT.BORDER);
		email.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true,
				false));

		Label phoneLabel = new Label(form, SWT.NONE);
		phoneLabel.setText("Phone");
		phoneLabel.setLayoutData(new GridData(GridData.BEGINNING,
				GridData.CENTER, false, false));
		phone = new Label(form, SWT.SINGLE | SWT.BORDER);
		phone.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true,
				false));

		Label streetLable = new Label(form, SWT.NONE);
		streetLable.setText("Street Address");
		streetLable.setLayoutData(new GridData(GridData.BEGINNING,
				GridData.CENTER, false, false));
		street = new Label(form, SWT.SINGLE | SWT.BORDER);
		street.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true,
				false));

		Label cityLabel = new Label(form, SWT.NONE);
		cityLabel.setText("City");
		cityLabel.setLayoutData(new GridData(GridData.BEGINNING,
				GridData.CENTER, false, false));
		city = new Label(form, SWT.SINGLE | SWT.BORDER);
		city.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true,
				false));

		Label zipcodeLabel = new Label(form, SWT.NONE);
		zipcodeLabel.setText("Zip Code");
		zipcodeLabel.setLayoutData(new GridData(GridData.BEGINNING,
				GridData.CENTER, false, false));
		zipcode = new Label(form, SWT.SINGLE | SWT.BORDER);
		zipcode.setLayoutData(new GridData(GridData.FILL, GridData.CENTER,
				true, false));

		Label countryLabel = new Label(form, SWT.NONE);
		countryLabel.setText("Country");
		countryLabel.setLayoutData(new GridData(GridData.BEGINNING,
				GridData.CENTER, false, false));
		country = new Label(form, SWT.SINGLE | SWT.BORDER);
		country.setLayoutData(new GridData(GridData.FILL, GridData.CENTER,
				true, false));

		Sash sash = new Sash(parent, SWT.HORIZONTAL);

		parent.setLayout(new FormLayout());

		form.pack();
		Point fsize = form.computeSize(SWT.DEFAULT, SWT.DEFAULT);

		FormData topData = new FormData();
		final FormData sashData = new FormData();
		FormData bottomData = new FormData();

		topData.left = new FormAttachment(0);
		topData.right = new FormAttachment(100);
		topData.top = new FormAttachment(0);
		topData.bottom = new FormAttachment(sash);

		sashData.left = new FormAttachment(0);
		sashData.right = new FormAttachment(100);
		sashData.top = new FormAttachment(100, -fsize.y);

		bottomData.left = new FormAttachment(0);
		bottomData.right = new FormAttachment(100);
		bottomData.top = new FormAttachment(sash);
		bottomData.bottom = new FormAttachment(100);

		table.setLayoutData(topData);
		sash.setLayoutData(sashData);
		form.setLayoutData(bottomData);

		sash.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				sashData.top = new FormAttachment(0, e.y);
				parent.layout();
			}
		});

		table.addSelectionListener(new SelectionAdapter() {

			/*
			 * (non-Javadoc) Handle double-click (contact editing)
			 */
			public void widgetDefaultSelected(SelectionEvent e) {
				int i = table.getSelectionIndex();
				if (i < 0)
					return;
				TableItem item = table.getItem(i);
				// TODO: edit selected contact.
			}

			/*
			 * (non-Javadoc) Handle contact selection
			 */
			public void widgetSelected(SelectionEvent e) {
				int i = table.getSelectionIndex();
				if (i < 0)
					return;
				TableItem item = table.getItem(i);
				// TODO: synchronize contact details with the selected contact
			}
		});

		table.setSelection(0);
	}
}
